import {
  Metadata,
  Loader,
  Http,
  IEventParam,
  IDictionary,
  I18n,
} from '@zeedhi/core';
import {
  Button,
  Report,
  LoadingService,
  ITabsBeforeChangeEventParam,
  Number as ZdNumber,
} from '@zeedhi/common';
import BudgetParameters from '@/controllers/BudgetParameters';
import { ZdTreeGrid, ZdTabs } from '@zeedhi/vuetify';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class CalculationOperatingBudgetController {
  public checkLine = false;
  
  public checkGridEdit = false;
  
  public checkGridDetailEdit = false;
  
  public filter;
  
  public titleBudget: string = 'Nenhum filtro aplicado.';
  
  public selectedCell: { rowId: string, column: string } = { rowId: '', column: '' };
  
  public clickedCell: { row: any, column: any, component: any } = { row: null, column: null, component: null };
  
  public activeGrid;
  
  public repeatValue = localStorage.getItem('repeatValueBudget_GOP') === 'true';
  
  public reportHeader = {
    YEAR: '',
    PERIOD: '',
    MANAGER: '',
  };
  
  private months: { [key: string]: number } = {
    JANEIRO: 1,
    FEVEREIRO: 2,
    MARCO: 3,
    ABRIL: 4,
    MAIO: 5,
    JUNHO: 6,
    JULHO: 7,
    AGOSTO: 8,
    SETEMBRO: 9,
    OUTUBRO: 10,
    NOVEMBRO: 11,
    DEZEMBRO: 12,
  };
  
  private requestOperationalItems: { [key: string]: string } = {
    gop10000_gridAdministrateTax: 'gop10000_mainGrid',
    gop10000_gridConsumption: 'gop10000_mainGrid',
    gop10000_gridDepreciation: 'gop10000_mainGrid',
    gop10000_gridEntryRelease: 'gop10000_mainGrid',
    gop10000_gridTaxRecovered: 'gop10000_mainGrid',
    gop10000_gridHeadquarters: 'gop10000_mainGrid',
    gop10000_gridInvestment: 'gop10000_mainGrid',
    gop10000_gridInvoicing: 'gop10000_mainGrid',
    gop10000_gridLabor: 'gop10000_mainGrid',
    gop10000_gridMiscellaneous: 'gop10000_mainGrid',
    gop10000_gridProfitTax: 'gop10000_mainGrid',
    gop10000_gridTurnoverTax: 'gop10000_mainGrid',
    gridSummary: 'gridSummary',
  };
  
  private mappingPage: { [key: string]: string } = {
    tabInvoicing: 'gop10000_gridInvoicing',
    tabTaxes: 'AGRUPIMP',
    tabTurnoverTax: 'gop10000_gridTurnoverTax',
    tabConsumption: 'gop10000_gridConsumption',
    tabExpense: 'AGRUPDESP',
    tabLabor: 'gop10000_gridLabor',
    tabAdministrateTax: 'gop10000_gridAdministrateTax',
    tabHeadquarters: 'gop10000_gridHeadquarters',
    tabMiscellaneous: 'gop10000_gridMiscellaneous',
    tabEntryRelease: 'gop10000_gridEntryRelease',
    tabTaxRecover: 'gop10000_gridTaxRecovered',
    tabProfitTax: 'gop10000_gridProfitTax',
    tabInvestment: 'gop10000_gridInvestment',
    tabDepreciation: 'gop10000_gridDepreciation',
  };
  
  public async mounted() {
    await BudgetParameters.getBudgetParameters();
    const filterOperatingBudgetModal = Loader.getInstance('FilterOperatingBudgetModalController');
    filterOperatingBudgetModal.createfilterOperatingBudgetModal();
    this.enableOperationalItem();
  }
  
  public defineTitleBudget() {
    let title = '';
    if (this.filter) {
      const year = this.filter.NRANOORCAGOP;
      const period = !this.filter.NRPERIODOORCA || this.filter.NRPERIODOORCA.IDPERIODOORCA === 'ANUAL'
        ? this.filter.IDPERIODOORCA : this.filter.NRPERIODOORCA.DSPERIODOORCA;
      const manager = BudgetParameters.getParameters.NMOPERADOR;
      title = I18n.translate('Ano').concat(': ', year, ' - ',
        I18n.translate('Período'), ': ', period, ' - ',
        I18n.translate('gop::Gerente Operacional'), ': ', manager);
      this.titleBudget = title;
      this.reportHeader = {
        YEAR: year,
        PERIOD: period,
        MANAGER: manager,
      };
    }
    this.initialize();
  }
  
  public tableData(component) {
    const utilities = new Utilities();
    let element;
    if (!component.name) {
      element = component.component;
    } else {
      element = component;
    }
    
    const routeName = this.requestOperationalItems[element.name];
    BudgetParameters.setRecharged(routeName);
    const filtered = BudgetParameters.getParametersHandled;
    const sendObject = {
      filter: [],
    };
    sendObject.filter.push(...filtered);
    
    let specificFilter;
    if (element.datasource.filter) {
      specificFilter = Object.entries(element.datasource.filter).map(([key, value]) => ({
        name: key,
        operator: '=',
        value,
      }));
    }
    sendObject.filter.push(...specificFilter);
    
    let data;
    
    LoadingService.show();
    Http.post(`/${routeName}`, sendObject).then((response) => {
      data = response.data.dataset[routeName];
      
      if (routeName === 'gridSummary' && data.some((obj) => ('IDTIPOORCAOP' in obj))) {
        data.forEach((obj) => {
          obj.IDTIPOORCAOP = I18n.translate('gop::'.concat(obj.IDTIPOORCAOP));
        });
      }
      element.datasource.setFilter([]);
      element.datasource.updateData(data);
      LoadingService.hide();
    }).catch((e) => {
      utilities.errorMessage('Falha ao buscar dados');
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  async sortMonths({ component }) {
    const startingMonth = parseInt(BudgetParameters.getParameters.NRMESINIORCA, 10);
    const visibleMonth = BudgetParameters.budgetMonths;
    let position = startingMonth - 1;
    let exclusionPosition;
    let janPosition = 0;
    
    Object.entries(this.months).forEach(([key, value]) => {
      const index = component.columns.findIndex((obj) => (obj.name === key));
      if (janPosition === 0) {
        janPosition = index;
      } else {
        janPosition += 1;
      }
      component.columns[index].isVisible = visibleMonth.includes(key);
      
      if (index !== -1 && position !== 0) {
        position += value === 1 ? index : 1;
        exclusionPosition = position + 1;
        if (component.columns[position].name !== 'TOTAL') {
          component.columns.splice(janPosition, 0, component.columns[position]);
          component.columns.splice(exclusionPosition, 1);
        } else {
          position = 0;
        }
      }
    });
    
    return component.columns;
  }
  
  public onInlineEditConditions({ component }: IEventParam<TekGrid>) {
    this.checkGridEdit = component.editing;
  }
  
  public onInlineDetailEditConditions({ component }: IEventParam<TekGrid>) {
    this.checkGridDetailEdit = component.editing;
  }
  
  public cellIsEditable({ row, column }: IDictionary<any>) {
    const month = this.months[column.name].toString().padStart(2, '0');
    let monthWithinLimit;
    let detailed = false;
    
    if (row.IDEDITMES) {
      const firstEditableMonth = parseInt(row.IDEDITMES.split('/').join(''), 10);
      const startingMonth = parseInt(BudgetParameters.getParameters.NRMESINIORCA, 10);
      const nextYear = parseInt(row.NRANOORCAGOP, 10) + 1;
      let budgetMonth;
      
      if (startingMonth > this.months[column.name]) {
        budgetMonth = parseInt(nextYear.toString().concat(month), 10);
      } else {
        budgetMonth = parseInt(row.NRANOORCAGOP.toString().concat(month), 10);
      }
      
      monthWithinLimit = (budgetMonth >= firstEditableMonth);
    } else {
      monthWithinLimit = false;
    }
    if (row.MESDETAL) {
      detailed = row.MESDETAL.split(',').includes(month);
    }
    return row.CONCLUSAO === 'PENDENTE' && !detailed && monthWithinLimit;
  }
  
  public async cancelEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    await grid.cancelEditedRows();
    this.onInlineEditConditions({ component: grid });
    this.rechargeBudget({ component: grid });
  }
  
  public async cancelDetailEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    await grid.cancelEditedRows();
    this.onInlineDetailEditConditions({ component: grid });
    this.rechargeBudget({ component: grid });
  }
  
  public async saveEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    const data = grid.getEditedRows();
    const utilities = new Utilities();
    
    LoadingService.show();
    Http.post('/changeBudgetMonth', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await grid.saveEditedRows();
      this.onInlineEditConditions({ component: grid });
      LoadingService.hide();
      
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public async saveDetailEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    const data = grid.getEditedRows();
    const utilities = new Utilities();
    
    LoadingService.show();
    Http.post('/changeBudgetDetailMonth', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await grid.saveEditedRows();
      this.onInlineDetailEditConditions({ component: grid });
      
      const gridNameMaster = grid.name.replace(/Detail$/, '');
      const gridMaster = Metadata.getInstance<TekGrid>(gridNameMaster);
      this.rechargeBudget({ component: gridMaster });
      
      this.clickedCellMark({
        row: this.clickedCell.row,
        column: this.clickedCell.column,
        component: this.clickedCell.component,
      });
      
      LoadingService.hide();
      
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public async completeBudget(data) {
    const utilities = new Utilities();
    
    LoadingService.show();
    Http.post('/completeBudget', data).then((response) => {
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      utilities.successMessage();
      LoadingService.hide();
      utilities.hideDialogue();
    }).catch((e) => {
      LoadingService.hide();
      utilities.errorMessage();
      console.warn(e);
    });
  }
  
  private budgetCalculation(data) {
    const utilities = new Utilities();
    
    LoadingService.show();
    Http.post('/budgetCalculation', data).then((response) => {
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      utilities.successMessage();
      LoadingService.hide();
      utilities.hideDialogue();
    }).catch((e) => {
      LoadingService.hide();
      utilities.errorMessage();
      console.warn(e);
    });
  }
  
  public confirmationServices({ component }) {
    const grid = component.parent as TekGrid;
    const selectedLines = grid.selectionState.except;
    const selectedUnits = selectedLines.map((obj) => (obj.CDFILIAL));
    const regexPercentage: RegExp = /percentage/i;
    const calcByPercentage: boolean = regexPercentage.test(component.name);
    const regexConclude: RegExp = /conclude/i;
    const budgetConclude: boolean = regexConclude.test(component.name);
    const regexPrevious: RegExp = /previous/i;
    const budgetPrevious: boolean = regexPrevious.test(component.name);
    const utilities = new Utilities();
    
    /*
      P: Calculation by percentage
      O: Standard budget calculation
    */
    let calculation;
    if (calcByPercentage) {
      calculation = 'P';
    } else if (budgetPrevious) {
      calculation = 'D';
    } else {
      calculation = 'O';
    }
    
    const data = {
      CDFILIAL: selectedUnits,
      CALCULATION: calculation,
      NRANOORCAGOP: selectedLines[0].NRANOORCAGOP,
      IDPERIODOORCA: selectedLines[0].IDPERIODOORCA,
      NRPERIODOORCA: selectedLines[0].NRPERIODOORCA,
      IDTIPOORCAOP: selectedLines[0].IDTIPOORCAOP,
      MESSAGE: '',
    };
    
    if (calcByPercentage) {
      data.MESSAGE = 'gop::CALC_ORCA_PERC';
    } else if (budgetConclude) {
      data.MESSAGE = I18n.translate('gop::CONC_ORCAMENTO');
    } else if (budgetPrevious) {
      data.MESSAGE = 'gop::CALC_ORCA_OLDORCA';
    } else {
      data.MESSAGE = '';
    }
    if (budgetConclude) {
      utilities.showDialog(this.completeBudget, data, 'YN');
    } else {
      utilities.showDialog(this.budgetCalculation, data, 'YN');
    }
  }
  
  public async rechargeBudget({ component }) {
    let grid;
    if (component.component === 'TekGrid') {
      grid = component;
    } else {
      grid = component.parent as TekGrid;
    }
    let filter;
    
    switch (grid.name) {
      case 'gop10000_gridAdministrateTax':
        filter = { IDTIPOORCAOP: 'TAXAADM' };
        break;
      case 'gop10000_gridConsumption':
        filter = { IDTIPOORCAOP: 'CONSUMO' };
        break;
      case 'gop10000_gridDepreciation':
        filter = { IDTIPOORCAOP: 'DEPRECIACAO' };
        break;
      case 'gop10000_gridEntryRelease':
        filter = { IDTIPOORCAOP: 'DESPENTRADA' };
        break;
      case 'gop10000_gridTaxRecovered':
        filter = { IDTIPOORCAOP: 'IMPOSTOARECUPERAR' };
        break;
      case 'gop10000_gridHeadquarters':
        filter = { IDTIPOORCAOP: 'DESPMATRIZ' };
        break;
      case 'gop10000_gridInvestment':
        filter = { IDTIPOORCAOP: 'INVESTIMENTO' };
        break;
      case 'gop10000_gridInvoicing':
        filter = { IDTIPOORCAOP: 'FATURAMENTO' };
        break;
      case 'gop10000_gridLabor':
        filter = { IDTIPOORCAOP: 'MAOOBRA' };
        break;
      case 'gop10000_gridMiscellaneous':
        filter = { IDTIPOORCAOP: 'DESPDIVERSA' };
        break;
      case 'gop10000_gridProfitTax':
        filter = { IDTIPOORCAOP: 'IMPOSTOSOBLUCRO' };
        break;
      case 'gop10000_gridTurnoverTax':
        filter = { IDTIPOORCAOP: 'IMPOSTO' };
        break;
      default:
        filter = '';
        break;
    }
    
    grid.datasource.filter = filter;
    this.tableData(grid);
  }
  
  public initialize() {
    const instance = Metadata.getInstance<ZdTreeGrid>('gridSummary');
    this.tableData(instance);
  }

  get disabledButtons() {
    return this.checkGridEdit || !this.checkLine;
  }
  
  get disabledPages() {
    return this.checkGridEdit || this.checkGridDetailEdit;
  }

  get disabledRecharge() {
    return this.checkGridEdit;
  }
  
  public disableSelection({ row }: IDictionary) {
    return row.CONCLUSAO !== 'PENDENTE';
  }
  
  public rowStyleConditions(row: IDictionary) {
    let lineColor = '';
    
    if (row.IDAPROVACAOGOP in ['REPROVADO', 'CANCELADO']) {
      lineColor = '#fffee1';
    }
    // ffdece vermelho
    // dceafc azul
    // cff6c8 verde efffe9
    // rosa fff0f6
    // amarelo fffee1
    return {
      cssStyle: { background: lineColor },
    };
  }
  
  public selectedLine({ component }) {
    this.checkLine = (component.selectionState.except.length > 0);
  }
  
  public unselectedLine({ component }) {
    this.checkLine = (component.selectionState.except.length > 0);
  }
  
  public enableOperationalItem() {
    const instance = Metadata.getInstance<ZdTabs>('tabsBudget');
    const itemOperaList = BudgetParameters.getParameters.ITEMOPERACIONAL;
    const mapping: { [key: string]: string } = {
      tabSummary: 'RESUMO',
      tabInvoicing: 'FATURAMENTO',
      tabTaxes: 'AGRUPIMP',
      tabTurnoverTax: 'IMPOSTO',
      tabConsumption: 'CONSUMO',
      tabExpense: 'AGRUPDESP',
      tabLabor: 'MAOOBRA',
      tabAdministrateTax: 'TAXAADM',
      tabHeadquarters: 'DESPMATRIZ',
      tabMiscellaneous: 'DESPDIVERSA',
      tabEntryRelease: 'DESPENTRADA',
      tabTaxRecover: 'IMPOSTOARECUPERAR',
      tabProfitTax: 'IMPOSTOSOBLUCRO',
      tabInvestment: 'INVESTIMENTO',
      tabDepreciation: 'DEPRECIACAO',
    };
    let daughterTab;
    instance.tabs.forEach((item) => {
      if (Array.isArray(item.children[0].tabs)) {
        daughterTab = false;
        item.children[0].tabs.forEach((subItem) => {
          subItem.isVisible = itemOperaList.includes(mapping[subItem.name]);
          if (!daughterTab) {
            daughterTab = subItem.isVisible;
          }
        });
        item.isVisible = daughterTab;
      } else {
        item.isVisible = itemOperaList.includes(mapping[item.name]);
        if (item.name === 'tabSummary') {
          item.isVisible = itemOperaList.includes(',');
        }
      }
    });
  }
  
  /**
  * Budget observations, a function designed to inform the current status of an operational
  * item in the budget of a given unit.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public budgetObservation({ row }: IEventParam<Button>) {
    const utilities = new Utilities();
    const data = {
      TITLE: I18n.translate('Unidade').concat(`: ${row.CDFILIAL} - ${row.NMFILIAL}`),
      MESSAGE: '',
    };
    
    switch (row.IDAPROVACAOGOP) {
      case 'PENDENTE':
        data.MESSAGE = I18n.translate('gop::PENDING_BUDGET_APPROVAL');
        break;
      case 'APROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_APPROVED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'REPROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_REJECTED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'CANCELADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_CANCELED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      default:
        data.MESSAGE = row.CONCLUSAO === 'PENDENTE'
          ? I18n.translate('gop::PENDING_BUDGET_CONC') : I18n.translate('gop::PENDING_APPROVAL');
        break;
    }
    return utilities.showDialog(null, data, 'OK');
  }
  
  /**
  * Standard model for exporting reports using zeedhi, in pdf and xls formats.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public async exportReport({ component }: IEventParam<Button>) {
    const grid = component.parent.parent as TekGrid;
    let title;
    
    switch (grid.name) {
      case 'gop10000_gridAdministrateTax':
        title = 'T_TAXAADM';
        break;
      case 'gop10000_gridConsumption':
        title = 'T_CONSUMO';
        break;
      case 'gop10000_gridDepreciation':
        title = 'T_DEPRECIACAO';
        break;
      case 'gop10000_gridEntryRelease':
        title = 'T_DESPENTRADA';
        break;
      case 'gop10000_gridTaxRecovered':
        title = 'T_IMPOSTOARECUPERAR';
        break;
      case 'gop10000_gridHeadquarters':
        title = 'T_DESPMATRIZ';
        break;
      case 'gop10000_gridInvestment':
        title = 'T_INVESTIMENTO';
        break;
      case 'gop10000_gridInvoicing':
        title = 'T_FATURAMENTO';
        break;
      case 'gop10000_gridLabor':
        title = 'T_MAOOBRA';
        break;
      case 'gop10000_gridMiscellaneous':
        title = 'T_DESPDIVERSA';
        break;
      case 'gop10000_gridProfitTax':
        title = 'T_IMPOSTOSOBLUCRO';
        break;
      case 'gop10000_gridTurnoverTax':
        title = 'T_IMPOSTO';
        break;
      default:
        title = '';
        break;
    }
    const report = new Report(grid, I18n.translate('gop::'.concat(title)));
    
    const reportHeader = {
      filter: [
        { label: I18n.translate('Ano'), value: this.reportHeader.YEAR },
        { label: I18n.translate('Período'), value: this.reportHeader.PERIOD },
        { label: I18n.translate('Gerente Operacional'), value: this.reportHeader.MANAGER },
      ],
    };
    
    const rowObj = {
      dataSet: {},
      filter: {},
      metaData: reportHeader,
    };

    const reportUrl = await report.getReport(component.label.toLowerCase(), false, rowObj);
    window.open(reportUrl);
  }
  
  public updateDataNecessary({ component, tabIndex }: ITabsBeforeChangeEventParam) {
    /*
    const mapping: { [key: string]: string } = {
      tabInvoicing: 'gop10000_gridInvoicing',
      tabTaxes: 'AGRUPIMP',
      tabTurnoverTax: 'gop10000_gridTurnoverTax',
      tabConsumption: 'gop10000_gridConsumption',
      tabExpense: 'AGRUPDESP',
      tabLabor: 'gop10000_gridLabor',
      tabAdministrateTax: 'gop10000_gridAdministrateTax',
      tabHeadquarters: 'gop10000_gridHeadquarters',
      tabMiscellaneous: 'gop10000_gridMiscellaneous',
      tabEntryRelease: 'gop10000_gridEntryRelease',
      tabTaxRecover: 'gop10000_gridTaxRecovered',
      tabProfitTax: 'gop10000_gridProfitTax',
      tabInvestment: 'gop10000_gridInvestment',
      tabDepreciation: 'gop10000_gridDepreciation',
    };
    */
    const tab = component.tabs[tabIndex].name;
    
    this.selectedCell.rowId = '';
    this.selectedCell.column = '';
    this.clickedCell.row = null;
    this.clickedCell.column = null;
    this.clickedCell.component = null;

    if (this.mappingPage[tab] && BudgetParameters.getNotRecharged(this.mappingPage[tab])) {
      const grid = Metadata.getInstance<TekGrid>(this.mappingPage[tab]);
      this.rechargeBudget({ component: grid });
    }
  }
  
  public repeatValueBudget() {
    this.repeatValue = !this.repeatValue;
  }

  public clickRepeatButtons() {
    this.repeatValue = !this.repeatValue;
    localStorage.setItem('repeatValueBudget_GOP', this.repeatValue ? 'true' : 'false');
  }

  get statusRepeatButtons() {
    return this.repeatValue;
  }

  get statusOffRepeatButtons() {
    return !this.repeatValue;
  }
  
  public async repeatBudget({ component, row }: IEventParam<ZdNumber>) {
    if (this.repeatValue) {
      const grid = component.parent as TekGrid;
      const { uniqueKey } = grid.datasource;
      const rowKey = row[uniqueKey];
      const month = component.name.split('_')[0];
      const nrOrcamentoGOP = grid.datasource.currentRow.NRORCAMENTOGOP;
      const dataRow = grid.getEditedRows().find((obj) => (obj.NRORCAMENTOGOP === nrOrcamentoGOP));
      const editedValue = dataRow[month];
      
      const sendObject = {
        MES: this.months[month],
        CDFILIAL: dataRow.CDFILIAL,
        EDITEDVALUE: editedValue,
        ANO: dataRow.NRANOORCAGOP,
      };
      LoadingService.show();
      
      Http.post('/workingDays', {
        requestType: 'Row',
        row: sendObject,
      }).then(async (response) => {
        const data = response.data.dataset.workingDays.dataset;
        
        const initialIndex = grid.columns.findIndex((obj) => obj.name === month) + 1;
        const endIndex = grid.columns.findIndex((obj) => obj.name === 'TOTAL');
        
        let value = 0;
        
        if (initialIndex !== -1 && endIndex !== -1) {
          for (let i = initialIndex; i <= endIndex; i += 1) {
            const obj = grid.columns[i];
  
            if (obj.editable) {
              value = data.find((item) => (item.MES === this.months[grid.columns[i].name])).VALOR;
              grid.updateCell(rowKey, grid.columns[i].name, value);
            }
          }
        }
        component.setFocus();
        LoadingService.hide();
      }).catch((e) => {
        LoadingService.hide();
        console.warn(e);
      });
    }
  }
  
  /* public getConditionalCellMark({ column }: any) {
    if (column.name !== this.selectedCell.column) return '';

    return 'selected-cell';
  } */
  
  public getConditionalCellMark({ row, column }: any) {
    if (row.CDFILIAL !== this.selectedCell.rowId || column.name !== this.selectedCell.column) return '';

    return 'selected-cell';
  }
  
  public getConditionalLineMark({ row }: any) {
    if (row.CDFILIAL !== this.selectedCell.rowId) return '';
    // const grid = component.parent as TekGrid;
    // if (row.CDFILIAL !== grid.datasource.currentRow.CDFILIAL) return '';

    return '';
  }
  
  /* public clickedCellMark({ column, component }: IEventParam<TekGrid>) {
    // this.selectedCell.rowId = row.CDFILIAL;
    this.selectedCell.column = column.name;
    // component.datasource.data.forEach((gridRow) => component.reapplyConditions(gridRow));
    this.reapplyCondition({ component });
  } */
  
  public clickedCellMark({ row, column, component }: IEventParam<TekGrid>) {
    this.activeGrid = component.name;
    if (row && column && this.months[column.name]) {
      this.selectedCell.rowId = row.CDFILIAL;
      this.selectedCell.column = column.name;
      this.reapplyCondition({ component });
      
      this.clickedCell.component = component;
      this.clickedCell.column = column;
      this.clickedCell.row = row;
      
      const typeOfBudget = row.IDTIPOORCAOP;
      
      if (row.MESDETAL && row.MESDETAL.split(',').includes(this.months[column.name].toString().padStart(2, '0'))) {
        const budgetNumber = row.NRORCAMENTOGOP;
        const budgetDate = this.months[column.name];
        
        this.tableDataDetail(budgetNumber, typeOfBudget, budgetDate);
      } else {
        this.tableDataDetail(null, typeOfBudget, null);
      }
    }
  }
  
  public async reapplyCondition({ component }: IEventParam<TekGrid>) {
    component.datasource.data.forEach((gridRow) => component.reapplyConditions(gridRow));
  }
  
  /**
  * Keyboard directional control, left key.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public async leftDirectionalKey(column) {
    const grid = column.component;
    if (this.activeGrid === grid.name) {
      const index = grid.columns.findIndex((obj) => obj.name === this.selectedCell.column);
      const nextIndex = index - 1;
      if (this.months[grid.columns[nextIndex].name] && nextIndex >= 0) {
        this.clickedCellMark({
          row: this.clickedCell.row,
          column: grid.columns[nextIndex],
          component: this.clickedCell.component,
        });
      }
    }
  }
  
  public async rightDirectionalKey(column) {
    const grid = column.component;
    if (this.activeGrid === grid.name) {
      const index = grid.columns.findIndex((obj) => obj.name === this.selectedCell.column);
      const nextIndex = index + 1;
      if (this.months[grid.columns[nextIndex].name] && nextIndex < grid.columns.length) {
        this.clickedCellMark({
          row: this.clickedCell.row,
          column: grid.columns[nextIndex],
          component: this.clickedCell.component,
        });
      }
    }
  }
  
  public async downDirectionalKey(row) {
    const grid = row.component;
    if (this.activeGrid === grid.name) {
      const index = grid.datasource.data.findIndex((obj) => obj.CDFILIAL === this.selectedCell.rowId);
      if (index !== grid.datasource.data.length) {
        const nextIndex = index + 1;
        this.clickedCellMark({
          row: grid.datasource.data[nextIndex],
          column: this.clickedCell.column,
          component: this.clickedCell.component,
        });
      }
    }
  }
  
  public async upDirectionalKey(row) {
    const grid = row.component;
    // const tab = Object.keys(this.mappingPage).find((key) => (this.mappingPage[key] === grid.name));
    if (this.activeGrid === grid.name) {
      const index = grid.datasource.data.findIndex((obj) => obj.CDFILIAL === this.selectedCell.rowId);
      if (index !== 0) {
        const nextIndex = index - 1;
        this.clickedCellMark({
          row: grid.datasource.data[nextIndex],
          column: this.clickedCell.column,
          component: this.clickedCell.component,
        });
      }
    }
  }

  /**
  * Function responsible for collecting data from detail grids directly related to the main grid.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public tableDataDetail(budgetNumber, typeOfBudget, budgetDate) {
    const utilities = new Utilities();
    const routeName = 'gop10000_gridDetail';
    
    const mapping: { [key: string]: string } = {
      INVESTIMENTO: 'gop10000_gridInvestmentDetail',
      IMPOSTOSOBLUCRO: 'gop10000_gridProfitTaxDetail',
      IMPOSTOARECUPERAR: 'gop10000_gridTaxRecoveredDetail',
      IMPOSTO: 'gop10000_gridTurnoverTaxDetail',
      DESPENTRADA: 'gop10000_gridEntryReleaseDetail',
      DESPDIVERSA: 'gop10000_gridMiscellaneousDetail',
      DESPMATRIZ: 'gop10000_gridHeadquartersDetail',
      MAOOBRA: 'gop10000_gridLaborDetail',
      CONSUMO: 'gop10000_gridConsumptionDetail',
      FATURAMENTO: 'gop10000_gridInvoicingDetail',
    };
    
    if (mapping[typeOfBudget]) {
      const grid = Metadata.getInstance<TekGrid>(mapping[typeOfBudget]);
      if (grid && budgetNumber) {
        const sendObject = {
          filter: [{
            name: 'NRORCAMENTOGOP',
            operator: '=',
            value: budgetNumber,
          },
          {
            name: 'IDTIPOORCAOP',
            operator: '=',
            value: typeOfBudget,
          },
          {
            name: 'DTORCAMENTO',
            operator: '=',
            value: budgetDate,
          }],
        };
        
        let data;
        
        LoadingService.show();
        Http.post(`/${routeName}`, sendObject).then((response) => {
          data = response.data.dataset[routeName];
          
          if (data.some((obj) => ('IDTIPO' in obj))) {
            data.forEach((obj) => {
              obj.IDTIPO = I18n.translate('gop::T_'.concat(obj.IDTIPO));
            });
          }
          grid.datasource.setFilter([]);
          
          grid.datasource.updateData(data);
          grid.parent.isVisible = true;
          grid.parent.parent.children[0].height = '50%';
          
          LoadingService.hide();
        }).catch((e) => {
          utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
          LoadingService.hide();
          console.warn(e);
        });
      } else {
        /* Detailing Grid */
        grid.parent.isVisible = false;
        grid.parent.parent.children[0].height = '99%';
        
        grid.datasource.setFilter([]);
        grid.datasource.updateData([]);
      }
    }
  }
  
  public budgetDismemberment({ row }: IEventParam<Button>) {
    const utilities = new Utilities();
    const data = {
      TITLE: I18n.translate('Unidade').concat(`: ${row.CDFILIAL} - ${row.NMFILIAL}`),
      MESSAGE: '',
    };
    
    switch (row.IDAPROVACAOGOP) {
      case 'PENDENTE':
        data.MESSAGE = I18n.translate('gop::PENDING_BUDGET_APPROVAL');
        break;
      case 'APROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_APPROVED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'REPROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_REJECTED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'CANCELADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_CANCELED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      default:
        data.MESSAGE = row.CONCLUSAO === 'PENDENTE'
          ? I18n.translate('gop::PENDING_BUDGET_CONC') : I18n.translate('gop::PENDING_APPROVAL');
        break;
    }
    return utilities.showDialog(null, data, 'OK');
  }
}
