import {
  Datasource,
  IEventParam,
  Metadata,
  Http,
} from '@zeedhi/core';
import {
  TekGrid,
} from '@zeedhi/teknisa-components-common';
import {
  Button,
  GridEditable,
  LoadingService,
  Text,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class PercentageAndStructureForBudgetController {
  // private departmentCurrentRow: any = {};
  
  private employeeCurrentRow: any = {};
  
  public isNotUpdateRowEditing = true;
  
  public isNotInlineSelectsWithRequestEditing = true;

  public onInlineEditUpdateRow({ component }: IEventParam<GridEditable>) {
    this.isNotUpdateRowEditing = !component.editing;
  }

  public async cancelEditingUpdateRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditUpdateRow({ component: grid });
    grid.editingNewRows = false;
  }

  public async saveEditingUpdateRow({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditUpdateRow({ component: grid });
    grid.editingNewRows = false;
  }

  get isInlineSelectsWithRequestEditing() {
    return !this.isNotInlineSelectsWithRequestEditing;
  }

  public onInlineEditSelectsWithRequest({ component }: IEventParam<GridEditable>) {
    this.isNotInlineSelectsWithRequestEditing = !component.editing;
  }

  public async cancelEditingSelectsWithRequest({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.cancelEditedRows();
    this.onInlineEditSelectsWithRequest({ component: grid });
  }

  public async saveEditingSelectsWithRequest({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    await grid.saveEditedRows();
    this.onInlineEditSelectsWithRequest({ component: grid });
  }
  
  public addNewRow({ component }: any) {
    const gridDetailItens = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    const grid: GridEditable = component.parent;
    const newRow = {
    };
    console.warn(gridDetailItens);
    grid.columns[2].editable = true;
    grid.editingNewRows = true;
    grid.addNewRow(newRow);
  }
  
  public structureItemChange({ component }: IEventParam<Button>) {
    const utilities = new Utilities();
    const grid = component.parent as TekGrid;
    const data = grid.getEditedRows();
    LoadingService.show();
    Http.post('/structureItemChange', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await grid.saveEditedRows();
      this.onInlineEditUpdateRow({ component: grid });
      LoadingService.hide();
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public unitRowClick({ component }: any) {
    const selectedUnit = `${component.datasource.currentRow.CDFILIAL} - ${component.datasource.currentRow.NMFILIAL}`;
    const grid = Metadata.getInstance<TekGrid>('gridOperationalItems');
    const unit = Metadata.getInstance<Text>('selectedUnit');
    
    grid.datasource.setFilter({
      CDFILIAL: component.datasource.currentRow.CDFILIAL,
    });
    
    unit.text = selectedUnit;
  }
  
  public rowOperationalItemClick({ component }: any) {
    const grid = Metadata.getInstance<TekGrid>('detailsOperationalItems');
    
    (grid as any).datasource.route = '/crudN/detailsOperationalItemsStructure';
    grid.datasource.setFilter({
      NRSEQITEMOP: component.datasource.currentRow.NRSEQ,
      CDFILIAL: component.datasource.currentRow.CDFILIAL,
    });
  }

  /*
  public onChangeDepartmentCurrentRow({ component }: IEventParam<Datasource>) {
    this.departmentCurrentRow = component.currentRow;
  }

  get hasNoDepartment() {
    return !this.departmentCurrentRow.CDFILIAL;
  }

  get hasDepartment() {
    return !!this.departmentCurrentRow.CDFILIAL;
  }
  */

  public onChangeEmployeeCurrentRow({ component }: IEventParam<Datasource>) {
    this.employeeCurrentRow = component.currentRow;
  }
}
